/*
 * A filesystem directory listing accessor.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

/* Instead of <stdint.h> for SIZE_MAX and <cstdint>... */

#ifndef SIZE_MAX
#define SIZE_MAX __SIZE_MAX__
#endif

#include <stdio.h>

#include <ext2fs/ext2fs.h>

#include <fsserver/accessor.h>
#include "fs_directory_listing.h"



/* Filesystem directory listing accessor. */

class Fs_directory_listing_accessor : public PagingAccessor
{
private:
  ext2_filsys _fs;
  ext2_ino_t _dir;
  int _flags;
  Fs_directory_listing *_listing = 0;

public:
  /* NOTE: The actual size of a directory listing is not readily determined. */

  explicit Fs_directory_listing_accessor(ext2_filsys fs, ext2_ino_t dir,
                                         int flags,
                                         Paging *paging, size_t size=SIZE_MAX)
  : PagingAccessor(paging, size), _fs(fs), _dir(dir), _flags(flags)
  {
  }

  /* Accessor methods. */

  unsigned int detach();

  unsigned long get_fileid() { return _dir; }

  long flush();

  size_t fstat(char *buffer, size_t length);

  size_t read(size_t start, char *buffer, size_t length);

  size_t write(size_t start, char *buffer, size_t length);

  /* Special methods. */

  long create_file(const char *name, uid_t uid, gid_t gid, mode_t umask,
                   ext2_ino_t *ino_file);
};
